from langchain_community.tools import WikipediaQueryRun
from langchain_community.utilities import WikipediaAPIWrapper

class WikipediaTool:
    def __init__(self):
        """Initialize the wikipedia tool."""
        self._client = WikipediaQueryRun(api_wrapper=WikipediaAPIWrapper())
        
    def run(self, query: str = None, language : str = "en", numResults: int = 5, max_chars_per_page: int = 4000):
        if query:
            self._client.api_wrapper.top_k_results = numResults
            self._client.api_wrapper.lang=language
            self._client.api_wrapper.doc_content_chars_max=max_chars_per_page
            result = self._client.run(query)
            return result




from deepeval import evaluate
from deepeval.metrics import (
    AnswerRelevancyMetric,
    ContextualPrecisionMetric,
    ContextualRecallMetric,
    ContextualRelevancyMetric,
    GEval,
    FaithfulnessMetric,
    HallucinationMetric,
    SummarizationMetric,
)
from deepeval.test_case import LLMTestCase
from deepeval.models.base_model import DeepEvalBaseLLM
from fabriq.llm_model import LLMModel
import traceback

class _CustomModel(DeepEvalBaseLLM):
    def __init__(self, config):
        self.config = config
        self.model = LLMModel(config).llm_model

    def load_model(self):
        return self.model

    def generate(self, prompt: str):
        resp = self.model.invoke(prompt).content
        return resp

    async def a_generate(self, prompt: str):
        resp = await self.model.ainvoke(prompt)
        return resp.content

    def get_model_name(self):
        return self.model.name


class Evaluation:
    def __init__(self, config):
        self.config = config
        self.model = _CustomModel(config)

    def run_evaluation(self, test_cases: list[LLMTestCase], metrics: list[str]):
        results = evaluate(test_cases=test_cases, metrics=metrics)
        return results

    def get_metrics(self):
        return [
            AnswerRelevancyMetric(),
            ContextualPrecisionMetric(),
            ContextualRecallMetric(),
            ContextualRelevancyMetric(),
            GEval(),
            FaithfulnessMetric(),
            HallucinationMetric(),
            SummarizationMetric(),
        ]

    def rag_evaluation(
        self,
        retrieved_docs: list[str],
        query: str = None,
        answer: str = None,
        expected_answer: str = None,
    ):
        metrics = [
            ContextualPrecisionMetric(model=self.model),
            ContextualRecallMetric(model=self.model),
            ContextualRelevancyMetric(model=self.model),
            HallucinationMetric(model=self.model),
        ]
        test_case = LLMTestCase(
            input=query,
            actual_output=answer,
            expected_output=expected_answer,
            retrieval_context=retrieved_docs,
            context=retrieved_docs,
        )

        results = {}
        for metric in metrics:
            try:
                metric_result = evaluate(test_cases=[test_case], metrics=[metric])
                results[metric.__class__.__name__] = {
                    "Score": metric_result.test_results[0].metrics_data[0].score,
                    "Reason": metric_result.test_results[0].metrics_data[0].reason,
                }
            except Exception as e:
                results[metric.__class__.__name__] = (
                    f"Skipped (insufficient parameters): {e}"
                )
                print(traceback.format_exc())
        return results



from pydantic import Field, BaseModel
from typing import List, Dict
import json


class ResponseFormat(BaseModel):
    categories: str = Field(description="a category for text to be classified.")


class ResponseFormat_MultiClass(BaseModel):
    categories: List[str] = Field(
        description="List of categories in which text can be classified."
    )


class TextClassification:
    def __init__(self, llm_model=None):
        """Initialize the Text Classification tool."""
        self.llm_model = llm_model
        self._multiclass_prompt = """You are an expert text classification tool. Classify the given text into the following categories:
            {categories}

            Text:
            {text}

            Follow this output format stricly:
            {{
            "categories" : ["<category_1>","<category_2>","<category_3>", ...]
            }}
            """
        self._singleclass_prompt = """You are an expert text classification tool. Classify the given text in strictly one of the following categories:
            {categories}

            Text:
            {text}

            Follow this output format stricly:
            {{
            "categories" : ["<category>"]
            }}
            """
        if not self.llm_model:
            raise ValueError(
                "Missing Required Parameter: 'llm_model'. Please pass LLM model to init method."
            )

    def run(
        self, text: str = None, categories: List[str] = [], multiclass: bool = False
    ):
        if text and len(categories) > 0:
            if multiclass:
                prompt = self._multiclass_prompt.format(
                    text=text, categories=categories
                )
                categories = self.llm_model.generate(prompt)
            else:
                prompt = self._singleclass_prompt.format(
                    text=text, categories=categories
                )
                categories = self.llm_model.generate(prompt)
            categories = json.loads(categories)

            return categories

        else:
            raise ValueError("Missing one or more parameters: text, categories.")



from fabriq.vector_store import VectorStore
from langchain.schema import Document
from typing import List
from collections import defaultdict


class VectorRetriever:
    def __init__(self, config):
        """Initialize the retriever with a vector store."""
        self.config = config
        self.vector_store = VectorStore(self.config)

    def reciprocal_rank_fusion(
        self, documents_list: List[List[Document]], k: int = 60
    ) -> List[Document]:
        """
        Apply Reciprocal Rank Fusion (RRF) to rerank documents from multiple lists.
        Since we don't have scores, we use the rank positions.
        Args:
            documents_list: List of lists of Documents (each list is a ranked list from a retriever).
            k: RRF constant (default 60).
        Returns:
            List[Document]: Reranked documents by RRF score.
        """

        doc_scores = defaultdict(float)
        doc_instances = {}

        for docs in documents_list:
            for rank, doc in enumerate(docs):
                doc_id = getattr(doc, "id", id(doc))
                doc_scores[doc_id] += 1.0 / (k + rank + 1)
                # Store the first instance of the doc for output
                if doc_id not in doc_instances:
                    doc_instances[doc_id] = doc

        # Sort by RRF score descending
        reranked_docs = sorted(
            doc_instances.values(),
            key=lambda d: doc_scores[getattr(d, "id", id(d))],
            reverse=True,
        )
        return reranked_docs

    def retrieve(
        self, query: str, top_k: int = 5, filter=None, rank_fusion: bool = False
    ) -> List[Document]:
        """Retrieve relevant documents based on the query."""
        documents = self.vector_store.retrieve(query, k=top_k, filter=filter)
        if rank_fusion:
            documents = self.reciprocal_rank_fusion([documents], k=60)
            # Ensure documents are unique and sorted by their RRF score
            unique_documents = {getattr(doc, "id", id(doc)): doc for doc in documents}
            documents = list(unique_documents.values())
            documents.sort(key=lambda d: getattr(d, "score", 0), reverse=True)

        if not documents:
            return None

        return documents



from langchain_community.tools import YouTubeSearchTool

class YoutubeSearch:
    def __init__(self):
        """Initialize the yahoo finance news tool."""
        self.client = YouTubeSearchTool()

    def run(self, query: str = None, numResults: int = 5):
        if query:
            query = f"{query},{numResults}"
            result = self.client.run(query)
            return result



from langchain.schema import Document
from langchain.chains.summarize import load_summarize_chain
from typing import List

class TextSummarization:
    def __init__(self, llm_model=None, summary_prompt: str = None):
        """Initialize the Text Summarization tool."""
        self.llm_model = llm_model
        if not self.llm_model:
            raise ValueError(
                "Missing Required Parameter: 'llm_model'. Please pass LLM model to init method."
            )
        self.summary_prompt = (
            summary_prompt
            if summary_prompt
            else """You are an expert text summarizer who excels in writing concise summary from any kind of text 
                while retaining key informations and insights from it. Summarize the given text.
                
                Text:
                {text}"""
        )

    def summarize_text(self, text: str = None):
        if text:
            prompt = self.summary_prompt.format(text=text)
            summary = self.llm_model.generate(prompt)
            return summary
        else:
            raise ValueError("Missing parameter: 'text'.")
        

    def summarize_docs(self, documents: List[str] = None):
        if documents or len(documents)>0:
            docs = [Document(page_content=doc) for doc in documents]
            chain = load_summarize_chain(llm=self.llm_model,chain_type="map_reduce")
            summary = chain.invoke(docs)
            return summary['output_text']

        else:
            raise ValueError("Missing list of documents to be passed to 'docs'.")
        
    def summarize_large_docs(self, documents: List[str] = None):
        if documents or len(documents)>0:
            docs = [Document(page_content=doc) for doc in documents]
            chain = load_summarize_chain(llm=self.llm_model,chain_type="refine")
            summary = chain.invoke(docs)
            return summary['output_text']
        
        else:
            raise ValueError("Missing list of documents to be passed to 'docs'.")
        
    def run(self, document: str | List[str] = None, summary_type: str = "text"):
        if summary_type == "text":
            return self.summarize_text(document)
        
        elif summary_type == "docs":
            return self.summarize_docs(document)
        
        elif summary_type == "large_docs":
            return self.summarize_large_docs(document)
        
        else:
            raise ValueError(f"Unsupported summary_type: {summary_type}. Possible values are: 'text', 'docs', 'large_docs'.")


from langchain_community.utilities.wolfram_alpha import WolframAlphaAPIWrapper

class WolframAlphaTool:
    def __init__(self):
        """Initialize the wolfram alpha tool."""
        self.client = WolframAlphaAPIWrapper()
        
    def run(self, query: str = None):
        if query:
            result = self.client.run(query)
            return result




from typing import List, Dict, Any
from ddgs import DDGS

class WebSearchTool:
    """A tool for performing web searches using DuckDuckGo."""
    def __init__(self):
        self._client = DDGS()

    def run(self, query: str) -> List[Dict[str, Any]]:
        """Search the web using DuckDuckGo and return the results."""
        try:
            search_results = []
            results = self._client.text(query, max_results=5,region="in-en")
            for result in results:
                search_results.append(result.get("body", ""))
            return search_results
        except Exception as e:
            # Handle rate limit or other exceptions gracefully
            return [{"error": f"Search failed: {str(e)}"}]


import pymupdf4llm
import markitdown
from unstructured.partition.auto import partition
import whisper
import openpyxl
import pandas as pd
from pptx import Presentation
from fabriq.document_loader.document_analyzer import BaseLoader, DocumentType, DocumentContent
from typing import Dict, Any
from tabulate import tabulate
from io import StringIO
from langchain.schema import Document
import os


class PyMuPDF4LLMLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        content = pymupdf4llm.to_markdown(
            file_path,
            page_chunks=True,
            embed_images=True,
            image_size_limit=0,
            force_text=False,
        )
        result = [
            Document(
                page_content=doc["text"],
                metadata={
                    "source": file_path,
                    "file_name": os.path.basename(file_path),
                    "page_num": doc["metadata"].get("page") or "",
                },
            )
            for doc in content
        ]
        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return doc_type == DocumentType.PDF and content.has_text


# class DoclingLoader(BaseLoader):
#     def load(self, file_path: str) -> Dict[str, Any]:
#         converter = DocumentConverter()
#         result = converter.convert(file_path)
#         return {"content": result.document.export_to_markdown(), "format": "markdown"}

#     def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
#         return doc_type == DocumentType.PDF


class UnstructuredLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        elements = partition(
            filename=file_path,
            strategy="hi_res",
            include_page_breaks=True,
            skip_infer_table_types=[],
            languages=["eng", "lat"],
        )
        # content = "\n\n".join([str(el) for el in elements])
        buffer = ""
        page_numbers = []
        text = []
        current_page_num = None

        last_page_num = None
        for el in elements:
            if el.metadata.page_number is not None:
                current_page_num = el.metadata.page_number
                last_page_num = current_page_num
            else:
                current_page_num = last_page_num

            if el.category == "Table":
                dfs = pd.read_html(StringIO(str(el.metadata.text_as_html)))
                if isinstance(dfs, list):
                    df = pd.concat(dfs, ignore_index=True)
                else:
                    df = dfs.reset_index(drop=True)
                table_text = tabulate(
                    df, headers="keys", tablefmt="github", showindex=False
                )
                buffer += "\n\n" + table_text + "\n\n"
            elif el.category == "Header":
                buffer += f"\n\n## {el.text.strip()}\n\n"
            elif el.category == "Title":
                buffer += f"\n\n# {el.text.strip()}\n\n"
            elif el.category == "ListItem":
                list_text = (
                    el.text.replace("● ", "- ")
                    .replace("• ", "- ")
                    .replace("○ ", "- ")
                    .strip()
                )
                buffer += f"{list_text}\n"
            elif el.category == "PageBreak":
                text.append(buffer.strip())
                page_numbers.append(current_page_num)
                buffer = ""
            else:
                buffer += el.text + "\n"

        if buffer.strip():
            text.append(buffer.strip())
            page_numbers.append(current_page_num)

        result = [
            Document(
                page_content=content,
                metadata={
                    "source": file_path,
                    "file_name": os.path.basename(file_path),
                    "page_num": page_num,
                },
            )
            for page_num, content in zip(page_numbers, text)
        ]

        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return doc_type == DocumentType.PDF and content.has_text


class MarkItDownLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        md = markitdown.MarkItDown()
        content = md.convert(file_path,keep_data_uris=True)
        result = Document(
            page_content=content.text_content,
            metadata={
                "source": file_path,
                "file_name": os.path.basename(file_path),
                # "page_num": page_num + 1,
            },
        )

        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return (
            doc_type in [DocumentType.DOCX, DocumentType.XLSX, DocumentType.XLS]
            and content.has_text
        )


class WhisperLoader(BaseLoader):
    def __init__(self):
        self.model = whisper.load_model("base")

    def load(self, file_path: str) -> Dict[str, Any]:
        content = self.model.transcribe(file_path)
        result = Document(
            page_content=content["text"],
            metadata={
                "source": file_path,
                "file_name": os.path.basename(file_path),
                "segment": content.get("segment", ""),
            },
        )

        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return doc_type in [
            DocumentType.MP3,
            DocumentType.WAV,
            DocumentType.M4A,
            DocumentType.MP4,
            DocumentType.AVI,
            DocumentType.MKV,
        ]


class PandasLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        if file_path.endswith(".csv"):
            df = pd.read_csv(file_path)
        elif file_path.endswith((".xlsx", ".xls")):
            df = pd.read_excel(file_path)
        else:
            raise ValueError(f"Unsupported file type for pandas loader: {file_path}")

        result = Document(
            page_content=df.to_markdown(index=False),
            metadata={
                "source": file_path,
                "file_name": os.path.basename(file_path),
            },
        )
        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return (
            doc_type in [DocumentType.CSV, DocumentType.XLSX, DocumentType.XLS]
            and content.has_text
        )


class OpenPyXLLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        wb = openpyxl.load_workbook(file_path)
        content = []
        for sheet_name in wb.sheetnames:
            sheet = wb[sheet_name]
            content.append(f"## {sheet_name}")
            for row in sheet.iter_rows(values_only=True):
                content.append(
                    " | ".join([str(cell) if cell is not None else "" for cell in row])
                )
        result = Document(
            page_content="\n".join(content),
            metadata={
                "source": file_path,
                "file_name": os.path.basename(file_path),
            },
        )
        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return doc_type in [DocumentType.XLSX, DocumentType.XLS] and content.has_text


# class PythonDocxLoader(BaseLoader):
#     def load(self, file_path: str) -> Dict[str, Any]:
#         doc = DocxDocument(file_path)
#         content = []
#         for paragraph in doc.paragraphs:
#             content.append(paragraph.text)
#         result = Document(
#             page_content="\n".join(content),
#             metadata={
#                 "source": file_path,
#                 "file_name": os.path.basename(file_path),
#             },
#         )
#         return result

#     def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
#         return doc_type == DocumentType.DOCX and content.has_text


class PPTXLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        prs = Presentation(file_path)
        content = []
        for i, slide in enumerate(prs.slides):
            content.append(f"## Slide {i+1}")
            for shape in slide.shapes:
                if hasattr(shape, "text"):
                    content.append(shape.text)
        result = Document(
            page_content="\n".join(content),
            metadata={
                "source": file_path,
                "file_name": os.path.basename(file_path),
            },
        )
        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return doc_type == DocumentType.PPTX and content.has_text


class SimpleTextLoader(BaseLoader):
    def load(self, file_path: str) -> Dict[str, Any]:
        with open(file_path, "r", encoding="utf-8") as f:
            content = f.read()
        result = Document(
            page_content=content,
            metadata={
                "source": file_path,
                "file_name": os.path.basename(file_path),
            },
        )
        return result

    def can_handle(self, doc_type: DocumentType, content: DocumentContent) -> bool:
        return (
            doc_type in [DocumentType.TXT, DocumentType.JSON, DocumentType.XML]
            and content.has_text
        )



from fabriq.llm_model import LLMModel
from fabriq.retriever import VectorRetriever
from fabriq.reranking import Reranker
from typing import List, Dict, Any
from langchain.schema import Document


class RAGPipeline:
    def __init__(self,config):
        """Initialize the RAG pipeline with an LLM model, retriever, and prompt template."""
        self.config = config
        self.llm_model = LLMModel(self.config)
        self.retriever = VectorRetriever(self.config)
        self.reranker = Reranker(self.config)
        # self.prompt_template = "You are a helpful AI assistant. Based on the following context, answer the question. DO NOT MAKE UP YOUR OWN ANSWER, ANSWER USING THE GIVEN CONTEXT ONLY. \n\n{documents}\n\nQuestion: {query}\nAnswer:"
        self.prompt_template : str = self.config.get("prompts").get("params").get("rag_prompt", None)

    def is_query_relevant(self, query: str, documents: List[Document]) -> bool:
        """Check if the query is relevant to the retrieved documents."""
        self.llm_model.system_prompt = "Your task is to determine if the query is relevant to the provided documents. Respond with 'True' or 'False' strictly."
        prompt = self.llm_model.system_prompt + f"\n\nDocuments: {documents}\n\nQuery: {query}"
        response = self.llm_model.generate(prompt)
        return True if "true" in response.lower() else False

    def get_response(self, query: str, filter=None, rerank: bool = False) -> str:
        """Run the RAG pipeline to retrieve relevant documents and generate a response."""
        # Retrieve relevant documents
        top_k = self.config.get("retriever").get("params").get("top_k", 25)
        documents = self.retriever.retrieve(query, top_k=top_k, filter=filter)
        if rerank:
            documents = self.reranker.rerank(query, documents)
        documents = documents[:top_k]

        if not documents:
            return "No relevant documents found."

        # Format the retrieved documents for the prompt
        documents = "\n\n----------\n\n".join([doc.page_content for doc in documents])

        if not self.prompt_template:
            raise ValueError("Prompt is not set.")
        if not self.llm_model:
            raise ValueError("LLM model is not initialized.")
        
        # Check if the query is relevant to the retrieved documents
        if self.is_query_relevant(query, documents) is False:
            result = {
                "text": "I cannot find relevant information to answer your question. Please ask your question relevant to the documents or rephrase it.",
                "chunks": [],
                "metadata": []
            }

        else:
            # Prepare the prompt with retrieved documents
            prompt = self.prompt_template.format(query=query, context=documents)

            # Generate a response using the LLM model
            response = self.llm_model.generate(prompt)

            result = {
                "text": response if response else "No response generated",
                "chunks": documents,
                "metadata": (
                    [getattr(doc, "metadata", {}) for doc in documents] if response else []
                ),
            }
        return result



from unstructured.partition.text import partition_text
from unstructured.chunking.basic import chunk_elements
from unstructured.chunking.title import chunk_by_title
from langchain_text_splitters import RecursiveCharacterTextSplitter
from langchain.schema import Document
from typing import List


class TextSplitter:
    def __init__(self, config):
        """
        Initializes the TextSplitter with the specified parameters.

        Args:
            splitter_type (str): The type of text splitter to use (default is "markdown").
            chunking_strategy (str): The strategy for chunking text (default is "by_title").
            chunk_size (int): The maximum size of each text chunk (default is 1000).
            chunk_overlap (int): The number of overlapping characters between chunks (default is 200).
        """
        self.config = config
        self.splitter_type = self.config.get("text_splitter").get(
            "type", "unstructured"
        )
        self.chunking_strategy = (
            self.config.get("text_splitter")
            .get("params")
            .get("chunking_strategy", "by_title")
        )
        self.chunk_size = (
            self.config.get("text_splitter").get("params").get("chunk_size", 1000)
        )
        self.chunk_overlap = (
            self.config.get("text_splitter").get("params").get("chunk_overlap", 200)
        )
        if self.splitter_type not in ["recursive", "unstructured"]:
            raise ValueError(
                f"Unsupported splitter type: {self.splitter_type}. "
                "Supported types are 'recursive' and 'unstructured'."
            )

    def split_text(self, documents: List[Document]) -> List[Document]:
        """
        Splits the input text into smaller chunks based on the specified splitting strategy.

        Args:
            text (str): The text to be split.

        Returns:
            List[Document]: A list of Document objects containing the split text chunks.
        """
        if self.splitter_type == "recursive":
            chunks = RecursiveCharacterTextSplitter(
                chunk_size=100,
                chunk_overlap=20,
            ).split_documents(documents)
            return chunks

        elif self.splitter_type == "unstructured":
            elements = []
            metadatas = []
            for text in documents:
                element = partition_text(text=text.page_content)
                elements.extend(element)
                metadatas.append(
                    {
                        "page_number": text.metadata.get("page_num", ""),
                        "filename": text.metadata.get("file_name", ""),
                    }
                )
            if self.chunking_strategy == "by_title":
                chunks = chunk_by_title(
                    elements,
                    multipage_sections=True,
                    overlap=self.chunk_overlap,
                    max_characters=self.chunk_size,
                )
            else:
                chunks = chunk_elements(
                    elements, overlap=self.chunk_overlap, max_characters=self.chunk_size
                )

            return [
                Document(
                    page_content=chunk.text,
                    metadata=metadata,
                )
                for chunk, metadata in zip(chunks, metadatas)
            ]



from setuptools import setup, find_packages

with open("requirements.txt", "r") as file:
    common_requirements = [line.strip() for line in file]

with open("packages/doc-loader.txt", "r") as file:
    doc_loader = [line.strip() for line in file]

with open("packages/index.txt", "r") as file:
    index = [line.strip() for line in file]
with open("packages/tools.txt", "r") as file:
    tools = [line.strip() for line in file]
with open("packages/eval.txt", "r") as file:
    eval = [line.strip() for line in file]
all = common_requirements + doc_loader + index + tools + eval

setup(
    name="fabriq",
    version="0.1.7",
    description="Fabriq is a Python SDK for developing quick, low code, and enterprise-level Generative AI solutions.",
    author="Aaryan Verma",
    url="https://github.com/Aaryanverma/fabriq",
    packages=find_packages(),
    install_requires=common_requirements,
    python_requires=">=3.10",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Microsoft :: Windows",
        "Operating System :: MacOS",
        "Operating System :: POSIX :: Linux",
    ],
    extras_require={
        'doc-loader': doc_loader,
        'index': index,
        'tools': tools,
        'eval': eval,
        'all': all
    }
)


from fabriq.vector_store import VectorStore
from fabriq.embeddings import EmbeddingModel
from fabriq.document_loader import SmartDocumentLoader
from fabriq.text_splitter import TextSplitter
from typing import List
import pandas as pd

class DocumentIndexer:
    def __init__(self, config):
        """Initialize the document indexer with a specified vector store type."""
        self.config = config
        self.embedding_model = EmbeddingModel(self.config)
        self.vector_store = VectorStore(self.config)
    def index_document(self, file_path: str):
        """Load a document and add it to the vector store."""
        loader = SmartDocumentLoader()
        splitter = TextSplitter(self.config)
        document = loader.load_document(file_path)
        chunks = splitter.split_text(document)
        self.vector_store.add_documents(chunks)

    def index_documents(self, file_paths: List[str]):
        """Index multiple documents."""
        error_files = pd.DataFrame(columns=["file_path", "error"])
        for file_path in file_paths:
            try:
                self.index_document(file_path)
            except Exception as e:
                print(f"Error indexing document {file_path}: {e}")
                error_files = pd.concat([error_files, pd.DataFrame({"file_path": [file_path], "error": [str(e)]})], ignore_index=True)
                error_files.to_excel("error_files.xlsx", index=False)